from __future__ import annotations
import ast
from typing import Tuple, TypeVar, Iterable
from func_adl import ObjectStream, func_adl_callback, func_adl_parameterized_call
import func_adl_servicex_xaodr24

_method_map = {
    'crate': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CMMJetHits_v1',
        'method_name': 'crate',
        'return_type': 'int',
    },
    'dataID': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CMMJetHits_v1',
        'method_name': 'dataID',
        'return_type': 'int',
    },
    'peak': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CMMJetHits_v1',
        'method_name': 'peak',
        'return_type': 'int',
    },
    'errorVec': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CMMJetHits_v1',
        'method_name': 'errorVec',
        'return_type_element': 'short',
        'return_type_collection': 'const vector<int>',
    },
    'hits': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CMMJetHits_v1',
        'method_name': 'hits',
        'return_type': 'unsigned int',
    },
    'error': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CMMJetHits_v1',
        'method_name': 'error',
        'return_type': 'int',
    },
    'Hits': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CMMJetHits_v1',
        'method_name': 'Hits',
        'return_type': 'unsigned int',
    },
    'Error': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CMMJetHits_v1',
        'method_name': 'Error',
        'return_type': 'int',
    },
    'index': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CMMJetHits_v1',
        'method_name': 'index',
        'return_type': 'int',
    },
    'usingPrivateStore': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CMMJetHits_v1',
        'method_name': 'usingPrivateStore',
        'return_type': 'bool',
    },
    'usingStandaloneStore': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CMMJetHits_v1',
        'method_name': 'usingStandaloneStore',
        'return_type': 'bool',
    },
    'hasStore': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CMMJetHits_v1',
        'method_name': 'hasStore',
        'return_type': 'bool',
    },
    'hasNonConstStore': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CMMJetHits_v1',
        'method_name': 'hasNonConstStore',
        'return_type': 'bool',
    },
    'clearDecorations': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CMMJetHits_v1',
        'method_name': 'clearDecorations',
        'return_type': 'bool',
    },
    'trackIndices': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CMMJetHits_v1',
        'method_name': 'trackIndices',
        'return_type': 'bool',
    },
    'auxdataConst': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CMMJetHits_v1',
        'method_name': 'auxdataConst',
        'return_type': 'U',
    },
    'isAvailable': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CMMJetHits_v1',
        'method_name': 'isAvailable',
        'return_type': 'bool',
    },
}


T = TypeVar('T')


def _add_method_metadata(s: ObjectStream[T], a: ast.Call) -> Tuple[ObjectStream[T], ast.Call]:
    '''Add metadata for a collection to the func_adl stream if we know about it
    '''
    assert isinstance(a.func, ast.Attribute)
    if a.func.attr in _method_map:
        s_update = s.MetaData(_method_map[a.func.attr])
        s_update = s_update.MetaData({
            'metadata_type': 'inject_code',
            'name': 'xAODTrigL1Calo/versions/CMMJetHits_v1.h',
            'body_includes': ["xAODTrigL1Calo/versions/CMMJetHits_v1.h"],
        })
        return s_update, a
    else:
        return s, a


@func_adl_callback(_add_method_metadata)
class CMMJetHits_v1:
    "A class"

    def crate(self) -> int:
        "A method"
        ...

    def dataID(self) -> int:
        "A method"
        ...

    def peak(self) -> int:
        "A method"
        ...

    def errorVec(self) -> func_adl_servicex_xaodr24.vector_int_.vector_int_:
        "A method"
        ...

    def hits(self) -> int:
        "A method"
        ...

    def error(self) -> int:
        "A method"
        ...

    def Hits(self) -> int:
        "A method"
        ...

    def Error(self) -> int:
        "A method"
        ...

    def index(self) -> int:
        "A method"
        ...

    def usingPrivateStore(self) -> bool:
        "A method"
        ...

    def usingStandaloneStore(self) -> bool:
        "A method"
        ...

    def hasStore(self) -> bool:
        "A method"
        ...

    def hasNonConstStore(self) -> bool:
        "A method"
        ...

    def clearDecorations(self) -> bool:
        "A method"
        ...

    def trackIndices(self) -> bool:
        "A method"
        ...

    @func_adl_parameterized_call(lambda s, a, param_1: func_adl_servicex_xaodr24.type_support.cpp_generic_1arg_callback('auxdataConst', s, a, param_1))
    @property
    def auxdataConst(self) -> func_adl_servicex_xaodr24.type_support.index_type_forwarder[str]:
        "A method"
        ...

    @func_adl_parameterized_call(lambda s, a, param_1: func_adl_servicex_xaodr24.type_support.cpp_generic_1arg_callback('isAvailable', s, a, param_1))
    @property
    def isAvailable(self) -> func_adl_servicex_xaodr24.type_support.index_type_forwarder[str]:
        "A method"
        ...
